from singleton import Singleton
import RPi.GPIO as GPIO
import spidev
from config import Config


class LowLevel(metaclass=Singleton):
    def __init__(self):
        self.__spi = spidev.SpiDev()
        self.__spi.open(0, 0)
        self.__spi.mode = 0
        self.__spi.max_speed_hz = 1000000
        self.__pwmu = None
        self.__pwml = None
        self.__cfg = Config()
        self.init()

    def init(self):
        GPIO.setmode(GPIO.BCM)
        sensors = self.__cfg.get_sensors_dict()
        for k in sensors:
            GPIO.setup(sensors[k]["BCM"], GPIO.OUT)
            GPIO.output(sensors[k]["BCM"], False)
        ufan = self.__cfg.get_pwm_upper()
        GPIO.setup(ufan["on-off"]["BCM"], GPIO.OUT)
        GPIO.output(ufan["on-off"]["BCM"], False)
        lfans = self.__cfg.get_pwm_lower()
        GPIO.setup(lfans["on-off"]["BCM"], GPIO.OUT)
        GPIO.output(lfans["on-off"]["BCM"], False)
        GPIO.setup(lfans["pwm"]["BCM"], GPIO.OUT)
        GPIO.setup(ufan["pwm"]["BCM"], GPIO.OUT)
        self.__pwmu = GPIO.PWM(ufan["pwm"]["BCM"], self.__cfg.get_pwm_defaults()["frequency"])
        self.__pwml = GPIO.PWM(lfans["pwm"]["BCM"], self.__cfg.get_pwm_defaults()["frequency"])
        self.__pwmu.start(0)
        self.__pwml.start(0)

    def __activate(self):
        self.__spi.cshigh = False

    def __deactivate(self):
        self.__spi.cshigh = True

    def retrieve_adc(self, channel):
        if channel > 7 or channel < 0:
            return None
        pin = self.__cfg.get_sensors_dict()[str(channel)]["BCM"]
        GPIO.output(pin, True)
        hchbit = channel >> 2
        lchbits = channel & 0x03
        to_send = [hchbit | 0x06, lchbits << 6, 0x00]
        self.__activate()
        r = self.__spi.xfer(to_send)
        self.__deactivate()
        GPIO.output(pin, False)
        highbyte = r[1]
        lowbyte = r[2]
        value = (highbyte << 8) | lowbyte
        return value

    def upper_fan(self, on):
        GPIO.output(self.__cfg.get_pwm_upper()["on-off"]["BCM"], on)

    def upper_fan_pwm(self, percent):
        self.__pwmu.ChangeDutyCycle(100 - percent)

    def lower_fan(self, on):
        GPIO.output(self.__cfg.get_pwm_lower()["on-off"]["BCM"], on)

    def lower_fan_pwm(self, percent):
        self.__pwml.ChangeDutyCycle(100 - percent)

